@@ -39,7 +39,7 @@
 
 #include "xnconfig.h"
 
-#define LIBRARY_VERSION_MAJOR		12
+#define LIBRARY_VERSION_MAJOR		15
 #define LIBRARY_VERSION_MINOR		0
 #define OPTIONS_DELIMETER		" "
 
@@ -52,20 +52,23 @@ static const char *status_names[] =	{"Disable", "Enable"};
 static const char *modifier_names[] =	{"Shift", "Control", "Alt", "Super"};
 
 static const char *option_names[] = 	{
-						"ManualMode", "ExcludeApp", "AddBind", "LogLevel", "ExcludeLanguage", "Autocomplementation",
-						"DisableCapsLock", "CheckOnProcess", "SetAutoApp", "SetManualApp", "AutocomplementationExcludeApp",
+						"ManualMode", "ExcludeApp", "AddBind", "LogLevel", "ExcludeLanguage", "Autocompletion",
+						"DisableCapsLock", "CheckOnProcess", "SetAutoApp", "SetManualApp", "AutocompletionExcludeApp",
 						"EducationMode", "Version", "LayoutRememberMode", "SaveSelectionMode",
 						"DefaultXkbGroup", "AddSound", "PlaySounds", "SendDelay", "LayoutRememberModeForApp",
 						"LogSave", "ReplaceAbbreviation",
 						"ReplaceAbbreviationIgnoreLayout", "CorrectIncidentalCaps", "CorrectTwoCapitalLetter",
 						"FlushBufferWhenPressEnter", "DontProcessWhenPressEnter", "AddAction",
 						"ShowOSD", "AddOSD", "FontOSD", "ShowPopup", "AddPopup", 
-	                    "CorrectSpaceWithPunctuation", "AddSpaceAfterAutocomplementation", "LoadModule",
+	                    "CorrectSpaceWithPunctuation", "AddSpaceAfterAutocompletion", "LoadModule",
 						"LogSize", "LogMail", "LogHostIP", "SoundVolumePercent",
 						"TroubleshootBackspace", "TroubleshootLeftArrow", "TroubleshootRightArrow",
-						"TroubleshootUpArrow", "TroubleshootDownArrow", "TroubleshootDelete", "TroubleshootSwitch",
+						"TroubleshootUpArrow", "TroubleshootDownArrow", "TroubleshootDelete", 
+						"TroubleshootSwitch", "TroubleshootFullScreen",
 						"DontSendKeyRelease", "LogPort", "RotateLayoutAfterChangeSelectedMode", "CorrectCapitalLetterAfterDot",
-						"FlushBufferWhenPressEscape", "CompatibilityWithCompletion"
+						"FlushBufferWhenPressEscape", "CompatibilityWithCompletion", "TrackingInput", "TrackingMouse",
+						"PopupExpireTimeout", "CorrectTwoSpaceWithCommaAndSpace","CorrectTwoMinusWithDash",
+						"CorrectCWithCopyright", "CorrectTMWithTrademark", "CorrectRWithRegistered"
 					};
 static const char *action_names[] =	{
 						"ChangeWord", "TranslitWord", "ChangecaseWord", "PreviewChangeWord",
@@ -73,7 +76,7 @@ static const char *action_names[] =	{
 						"ChangeSelected", "TranslitSelected", "ChangecaseSelected", "PreviewChangeSelected",
 						"ChangeClipboard", "TranslitClipboard", "ChangecaseClipboard", "PreviewChangeClipboard",
 						"EnableLayout1", "EnableLayout2", "EnableLayout3", "EnableLayout4",
-						"RotateLayout","RotateLayoutBack","ReplaceAbbreviation", "AutocomplementationConfirmation",
+						"RotateLayout","RotateLayoutBack","ReplaceAbbreviation", "AutocompletionConfirmation",
 						"BlockKeyboardAndMouseEvents", "InsertDate"
 					};
 static const char *notify_names[] =	{
@@ -84,8 +87,10 @@ static const char *notify_names[] =	{
 						"ChangeString",
 						"ChangeSelected", "TranslitSelected", "ChangecaseSelected", "PreviewChangeSelected",
 						"ChangeClipboard", "TranslitClipboard", "ChangecaseClipboard",  "PreviewChangeClipboard",
-						"ReplaceAbbreviation", "CorrectIncidentalCaps", "CorrectTwoCapitalLetter", "ExecuteUserAction", 
-						"BlockKeyboardAndMouseEvents", "UnblockKeyboardAndMouseEvents"
+						"ReplaceAbbreviation", "CorrectIncidentalCaps", "CorrectTwoCapitalLetter", 
+						"CorrectTwoSpaceWithCommaAndSpace", "CorrectTwoMinusWithDash", "CorrectCWithCopyright", 
+						"CorrectTMWithTrademark", "CorrectRWithRegistered",
+						"ExecuteUserAction","BlockKeyboardAndMouseEvents", "UnblockKeyboardAndMouseEvents"
 					};
 
 pid_t getsid(pid_t pid);
@@ -240,7 +245,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 				break;
 			}
 
-			p->autocomplementation = index;
+			p->autocompletion = index;
 			break;
 		}
 		case 6: // Disable CapsLock use
@@ -277,9 +282,9 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->manual_apps->add(p->manual_apps, full_string);
 			break;
 		}
-		case 10: // Autocomplementation Exclude App
+		case 10: // Autocompletion Exclude App
 		{
-			p->autocomplementation_excluded_apps->add(p->autocomplementation_excluded_apps, full_string);
+			p->autocompletion_excluded_apps->add(p->autocompletion_excluded_apps, full_string);
 			break;
 		}
 		case 11: // Get Education Mode
@@ -642,7 +647,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 			if (index == -1)
 				break;
 
-			p->add_space_after_autocomplementation = index;
+			p->add_space_after_autocompletion = index;
 			break;
 		}
 		case 35: // Add plugin 
@@ -756,7 +761,19 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->troubleshoot_switch = status;
 			break;
 		}
-		case 47: // Get dont send KeyRelease event mode
+		case 47: // Get troubleshooting mode for full screen apps
+		{
+			int status = get_option_index(bool_names, param);
+			if (status == -1)
+			{
+				log_message(WARNING, _("Invalid value for troubleshooting mode for full screen apps specified"));
+				break;
+			}
+
+			p->troubleshoot_full_screen = status;
+			break;
+		}
+		case 48: // Get dont send KeyRelease event mode
 		{
 			int status = get_option_index(bool_names, param);
 			if (status == -1)
@@ -768,14 +785,14 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->dont_send_key_release = status;
 			break;
 		}
-		case 48: // Port on host to send logfile
+		case 49: // Port on host to send logfile
 		{
 			p->port_keyboard_log = atoi(get_word(&param));
 			if ((p->port_keyboard_log < 1) || (p->port_keyboard_log > 32000))
 				p->port_keyboard_log = 25;
 			break;
 		}
-		case 49: // Rotate layout after convert selected text
+		case 50: // Rotate layout after convert selected text
 		{
 			int index = get_option_index(bool_names, param);
 			if (index == -1)
@@ -787,7 +804,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->rotate_layout_after_convert = index;
 			break;
 		}
-		case 50: // Correct small letter to capital letter after dot
+		case 51: // Correct small letter to capital letter after dot
 		{
 			int index = get_option_index(bool_names, param);
 			if (index == -1)
@@ -799,7 +816,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->correct_capital_letter_after_dot = index;
 			break;
 		}
-		case 51: // Flush internal buffer when pressed Escape Mode
+		case 52: // Flush internal buffer when pressed Escape Mode
 		{
 			int index = get_option_index(bool_names, param);
 			if (index == -1)
@@ -811,7 +828,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->flush_buffer_when_press_escape = index;
 			break;
 		}
-		case 52: // Compatibility with the completion
+		case 53: // Compatibility with the completion
 		{
 			int index = get_option_index(bool_names, param);
 			if (index == -1)
@@ -823,6 +840,100 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->compatibility_with_completion = index;
 			break;
 		}
+		case 54: // Tracking input
+		{
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+			{
+				log_message(WARNING, _("Invalid value for tracking input mode specified"));
+				break;
+			}
+
+			p->tracking_input = index;
+			break;
+		}
+		case 55: // Tracking mouse
+		{
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+			{
+				log_message(WARNING, _("Invalid value for tracking mouse mode specified"));
+				break;
+			}
+
+			p->tracking_mouse = index;
+			break;
+		}
+		case 56: // PopupExpireTimeout
+		{
+			p->popup_expire_timeout = atoi(param);
+			if (p->popup_expire_timeout < 0 || p->popup_expire_timeout > 30000)
+			{
+				log_message(WARNING, _("Popup expire timeout must be between 0 and 30000"));
+				p->popup_expire_timeout = 0;
+			}
+			break;
+		}
+		case 57: // CorrectTwoSpaceWithCommaAndSpace
+		{
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+			{
+				log_message(WARNING, _("Invalid value for correct two space with a comma and a space mode specified"));
+				break;
+			}
+
+			p->correct_two_space_with_comma_and_space = index;
+			break;
+		}
+		case 58: // CorrectTwoMinusWithDash
+		{
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+			{
+				log_message(WARNING, _("Invalid value for correct two minus with a dash mode specified"));
+				break;
+			}
+
+			p->correct_two_minus_with_dash = index;
+			break;
+		}
+		case 59: // CorrectCWithCopyright
+		{
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+			{
+				log_message(WARNING, _("Invalid value for correct (c) with a copyright sign mode specified"));
+				break;
+			}
+
+			p->correct_c_with_copyright = index;
+			break;
+		}
+		case 60: // CorrectTMWithTrademark
+		{
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+			{
+				log_message(WARNING, _("Invalid value for correct (tm) with a trademark sign mode specified"));
+				break;
+			}
+
+			p->correct_tm_with_trademark = index;
+			break;
+		}
+		case 61: // CorrectRWithRegistered
+		{
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+			{
+				log_message(WARNING, _("Invalid value for correct (r) with a registered sign mode specified"));
+				break;
+			}
+
+			p->correct_r_with_registered = index;
+			break;
+		}
 	}
 	free(full_string);
 }
@@ -862,7 +973,7 @@ static void free_structures(struct _xneur_config *p)
 	p->auto_apps->uninit(p->auto_apps);
 	p->layout_remember_apps->uninit(p->layout_remember_apps);
 	p->excluded_apps->uninit(p->excluded_apps);
-	p->autocomplementation_excluded_apps->uninit(p->autocomplementation_excluded_apps);
+	p->autocompletion_excluded_apps->uninit(p->autocompletion_excluded_apps);
 	p->abbreviations->uninit(p->abbreviations);
 	p->plugins->uninit(p->plugins);
 	
@@ -984,7 +1095,7 @@ static void xneur_config_clear(struct _xneur_config *p)
 	p->auto_apps			= list_char_init();
 	p->layout_remember_apps		= list_char_init();
 	p->excluded_apps		= list_char_init();
-	p->autocomplementation_excluded_apps	= list_char_init();
+	p->autocompletion_excluded_apps	= list_char_init();
 	p->abbreviations		= list_char_init();
 	p->plugins				= list_char_init();
 	
@@ -1202,6 +1313,31 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "#CorrectCapitalLetterAfterDot Yes\n");
 	fprintf(stream, "CorrectCapitalLetterAfterDot %s\n\n", p->get_bool_name(p->correct_capital_letter_after_dot));
 
+	fprintf(stream, "# This option enable or disable correction of two space with a comma and a space\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#CorrectTwoSpaceWithCommaAndSpace Yes\n");
+	fprintf(stream, "CorrectTwoSpaceWithCommaAndSpace %s\n\n", p->get_bool_name(p->correct_two_space_with_comma_and_space));
+
+	fprintf(stream, "# This option enable or disable correction of two minus with a dash\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#CorrectTwoMinusWithDash Yes\n");
+	fprintf(stream, "CorrectTwoMinusWithDash %s\n\n", p->get_bool_name(p->correct_two_minus_with_dash));
+
+	fprintf(stream, "# This option enable or disable correction of (c) with a copyright sign\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#CorrectCWithCopyright Yes\n");
+	fprintf(stream, "CorrectCWithCopyright %s\n\n", p->get_bool_name(p->correct_c_with_copyright));
+
+	fprintf(stream, "# This option enable or disable correction of (tm) with a trademark sign\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#CorrectTMWithTrademark Yes\n");
+	fprintf(stream, "CorrectTMWithTrademark %s\n\n", p->get_bool_name(p->correct_tm_with_trademark));
+
+	fprintf(stream, "# This option enable or disable correction of (r) with a registered sign\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#CorrectRWithRegistered Yes\n");
+	fprintf(stream, "CorrectRWithRegistered %s\n\n", p->get_bool_name(p->correct_r_with_registered));
+	
 	fprintf(stream, "# This option enable or disable flushing internal buffer when pressed Escape\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#FlushBufferWhenPressEscape Yes\n");
@@ -1241,6 +1377,11 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "#ShowPopup Yes\n");
 	fprintf(stream, "ShowPopup %s\n\n", p->get_bool_name(p->show_popup));
 
+	fprintf(stream, "# This option defines popup expiration interval in milliseconds\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#PopupExpireTimeout 1000\n");
+	fprintf(stream, "PopupExpireTimeout %d\n\n", p->popup_expire_timeout);
+
 	fprintf(stream, "# Binds popup messages for some actions\n");
 	for (int notify = 0; notify < MAX_NOTIFIES; notify++)
 	{
@@ -1265,23 +1406,23 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "#CorrectSpaceWithPunctuation No\n");
 	fprintf(stream, "CorrectSpaceWithPunctuation %s\n", p->get_bool_name(p->correct_space_with_punctuation));
 
-	fprintf(stream, "\n# This option disable or enable pattern mining and recognition (autocomplementation)\n");
+	fprintf(stream, "\n# This option disable or enable pattern mining and recognition (autocompletion)\n");
 	fprintf(stream, "# Example:\n");
-	fprintf(stream, "#Autocomplementation No\n");
-	fprintf(stream, "Autocomplementation %s\n", p->get_bool_name(p->autocomplementation));
+	fprintf(stream, "#Autocompletion No\n");
+	fprintf(stream, "Autocompletion %s\n", p->get_bool_name(p->autocompletion));
 
-	fprintf(stream, "\n# This option disable or enable adding space after autocomplementation\n");
+	fprintf(stream, "\n# This option disable or enable adding space after autocompletion\n");
 	fprintf(stream, "# Example:\n");
-	fprintf(stream, "#AddSpaceAfterAutocomplementation No\n");
-	fprintf(stream, "AddSpaceAfterAutocomplementation %s\n", p->get_bool_name(p->add_space_after_autocomplementation));
+	fprintf(stream, "#AddSpaceAfterAutocompletion No\n");
+	fprintf(stream, "AddSpaceAfterAutocompletion %s\n", p->get_bool_name(p->add_space_after_autocompletion));
 	fprintf(stream, "\n");
 
-	fprintf(stream, "# Add Applications names to exclude it from autocomplementation process\n");
-	fprintf(stream, "# Xneur will not process the autocomplementation for this applications\n");
+	fprintf(stream, "# Add Applications names to exclude it from autocompletion process\n");
+	fprintf(stream, "# Xneur will not process the autocompletion for this applications\n");
 	fprintf(stream, "# Example:\n");
-	fprintf(stream, "#AutocomplementationExcludeApp Gnome-terminal\n");
-	for (int i = 0; i < p->autocomplementation_excluded_apps->data_count; i++)
-		fprintf(stream, "AutocomplementationExcludeApp %s\n", p->autocomplementation_excluded_apps->data[i].string);
+	fprintf(stream, "#AutocompletionExcludeApp Gnome-terminal\n");
+	for (int i = 0; i < p->autocompletion_excluded_apps->data_count; i++)
+		fprintf(stream, "AutocompletionExcludeApp %s\n", p->autocompletion_excluded_apps->data[i].string);
 	fprintf(stream, "\n");
 
 	fprintf(stream, "# Use next options for troubleshoot on autoswitching\n");
@@ -1291,10 +1432,15 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "# Disable autoswitching if pressed up arrow\nTroubleshootUpArrow %s\n", p->get_bool_name(p->troubleshoot_up_arrow));
 	fprintf(stream, "# Disable autoswitching if pressed down arrow\nTroubleshootDownArrow %s\n", p->get_bool_name(p->troubleshoot_down_arrow));
 	fprintf(stream, "# Disable autoswitching if pressed delete\nTroubleshootDelete %s\n", p->get_bool_name(p->troubleshoot_delete));
-	fprintf(stream, "# Disable autoswitching if layout switched\nTroubleshootSwitch %s\n\n", p->get_bool_name(p->troubleshoot_switch));
-
+	fprintf(stream, "# Disable autoswitching if layout switched\nTroubleshootSwitch %s\n", p->get_bool_name(p->troubleshoot_switch));
+	fprintf(stream, "# Disable autoswitching for full screen apps\nTroubleshootFullScreen %s\n\n", p->get_bool_name(p->troubleshoot_full_screen));
+	
 	fprintf(stream, "# Work-arround for compatibility with the completion\nCompatibilityWithCompletion %s\n\n", p->get_bool_name(p->compatibility_with_completion));
 
+	fprintf(stream, "# Disabling this option will add any application to the list of excluded applications.\nTrackingInput %s\n\n", p->get_bool_name(p->tracking_input));
+
+	fprintf(stream, "# Disabling this option will disable mouse tracking.\nTrackingMouse %s\n\n", p->get_bool_name(p->tracking_mouse));
+
 	fprintf(stream, "# Disable send KeyRelease event\nDontSendKeyRelease %s\n\n", p->get_bool_name(p->dont_send_key_release));
 	
 	fprintf(stream, "# Modules list\n");
@@ -1346,6 +1492,7 @@ static void xneur_config_save_dict(struct _xneur_config *p, int lang)
 	char *lang_dir = (char *) malloc(path_len * sizeof(char));
 	snprintf(lang_dir, path_len, "%s/%s", LANGUAGEDIR, p->handle->languages[lang].dir);
 	save_list(p->handle->languages[lang].dictionary, lang_dir, DICT_NAME);
+	free (lang_dir);
 }
 
 static void xneur_config_save_pattern(struct _xneur_config *p, int lang)
@@ -1432,7 +1579,7 @@ struct _xneur_config* xneur_config_init(void)
 	p->layout_remember_apps		= list_char_init();
 	p->window_layouts		= list_char_init();
 	p->abbreviations		= list_char_init();
-	p->autocomplementation_excluded_apps	= list_char_init();
+	p->autocompletion_excluded_apps	= list_char_init();
 	p->plugins		= list_char_init();
 
 	p->troubleshoot_backspace = FALSE; 
@@ -1442,8 +1589,13 @@ struct _xneur_config* xneur_config_init(void)
 	p->troubleshoot_down_arrow = FALSE;
 	p->troubleshoot_delete = FALSE;
 	p->troubleshoot_switch = TRUE;
-
+	p->troubleshoot_full_screen = TRUE;
+	
 	p->dont_send_key_release = FALSE;
+	p->tracking_input = TRUE;
+	p->tracking_mouse = TRUE;
+	
+	p->popup_expire_timeout = 1000;
 
 	// Function mapping
 	p->get_home_dict_path		= get_home_file_path_name;
