@@ -26,7 +26,7 @@
 
 #define SOUNDDIR			"sounds"
 
-#define MAX_NOTIFIES		31
+#define MAX_NOTIFIES		36
 #define MAX_HOTKEYS			24
 
 #include "xneur.h"
@@ -69,6 +69,11 @@ enum _notify_action
 	NOTIFY_REPLACE_ABBREVIATION,
 	NOTIFY_CORR_INCIDENTAL_CAPS,
 	NOTIFY_CORR_TWO_CAPITAL_LETTER,
+	NOTIFY_CORR_TWO_SPACE,
+	NOTIFY_CORR_TWO_MINUS,
+	NOTIFY_CORR_COPYRIGHT,
+	NOTIFY_CORR_TRADEMARK,
+	NOTIFY_CORR_REGISTERED,
 	NOTIFY_EXEC_USER_ACTION,
 	NOTIFY_BLOCK_EVENTS,
 	NOTIFY_UNBLOCK_EVENTS,
@@ -98,7 +103,7 @@ enum _hotkey_action
 	ACTION_ROTATE_LAYOUT,
 	ACTION_ROTATE_LAYOUT_BACK,
 	ACTION_REPLACE_ABBREVIATION,
-	ACTION_AUTOCOMPLEMENTATION,
+	ACTION_AUTOCOMPLETION,
 	ACTION_BLOCK_EVENTS,
 	ACTION_INSERT_DATE,
 	ACTION_NONE,
@@ -125,6 +130,7 @@ enum _change_action
 	CHANGE_STRING_TO_LAYOUT_2,
 	CHANGE_STRING_TO_LAYOUT_3,
 	CHANGE_ABBREVIATION,
+	CHANGE_TWO_SPACE,
 	CHANGE_INS_DATE,
 };
 
@@ -194,21 +200,29 @@ struct _xneur_config
 	int   correct_incidental_caps;			// Change iNCIDENTAL CapsLock
 	int   correct_two_capital_letter;		// Change two CApital letter
 	int   correct_space_with_punctuation;	// Correct spaces before punctuation
+	int   correct_two_space_with_comma_and_space;	// Correct Two Space With A Comma And A Space
+	int   correct_capital_letter_after_dot;
+	int   correct_two_minus_with_dash;
+	int   correct_c_with_copyright;
+	int   correct_tm_with_trademark;
+	int   correct_r_with_registered;
+	
 	int   flush_buffer_when_press_escape;		// Flush internal buffer when pressed Escape
 	int   flush_buffer_when_press_enter;		// Flush internal buffer when pressed Enter or Tab
 	int   dont_process_when_press_enter;		// Don't correct word when pressed Enter
 	int   check_lang_on_process;			// Check lang on input process
 	int   disable_capslock;				// Disable CapsLock use
-	int   correct_capital_letter_after_dot;
+
 	int   block_events;					// Block all events (keyboard and mouse)
 	
-	int   autocomplementation;			// Save pattern and mining
-	int   add_space_after_autocomplementation;
-	struct _list_char *autocomplementation_excluded_apps;
+	int   autocompletion;			// Save pattern and mining
+	int   add_space_after_autocompletion;
+	struct _list_char *autocompletion_excluded_apps;
 
 	int   show_popup;				// Show popups
 	int   show_osd;					// Show OSD
 	char  *osd_font;
+	int   popup_expire_timeout;                     // Popup expire timeout in milliseconds
 
 	int   abbr_ignore_layout;			// Ignore keyboard layout for abbreviations
 
@@ -226,11 +240,14 @@ struct _xneur_config
 	int troubleshoot_down_arrow;
 	int troubleshoot_delete;
 	int troubleshoot_switch;
-
+	int troubleshoot_full_screen;
+	
 	int compatibility_with_completion;
 		
 	int dont_send_key_release;
-	
+	int tracking_input;
+	int tracking_mouse;
+		
 	char* (*get_home_dict_path) (const char *dir_name, const char *file_name);
 	char* (*get_global_dict_path) (const char *dir_name, const char *file_name);
 	const char* (*get_bool_name) (int option);
