@@ -33,6 +33,7 @@
 #endif
 
 #include "xneur.h"
+#include "window.h"
 
 #include "xnconfig_files.h"
 
@@ -45,6 +46,8 @@
 
 #include "log.h"
 
+extern struct _window *main_window;
+
 struct _xneur_config *xconfig				= NULL;
 
 #if  defined(WITH_ASPELL) || defined(WITH_ENCHANT)
@@ -188,8 +191,10 @@ struct _xneur_handle *xneur_handle_create (void)
 			continue;
 
 		char *group_name = XGetAtomName(display, group_atom);
+		//log_message (ERROR, "%s", group_name);
 		char *short_name = strsep(&prop_value, ",");
-
+		//log_message (ERROR, "%s", short_name);
+		
 		handle->languages = (struct _xneur_language *) realloc(handle->languages, (handle->total_languages + 1) * sizeof(struct _xneur_language));
 		bzero(&(handle->languages[handle->total_languages]), sizeof(struct _xneur_language));
 
@@ -199,9 +204,11 @@ struct _xneur_handle *xneur_handle_create (void)
 		handle->languages[handle->total_languages].dir[2] = NULLSYM;
 		handle->languages[handle->total_languages].group	= group;
 		handle->languages[handle->total_languages].excluded	= FALSE;
+		handle->languages[handle->total_languages].disable_auto_detection	= FALSE;
 		handle->total_languages++;
 
-		free(group_name);
+		//if (group_name != NULL)
+			//free(group_name);
 		
 		if (prop_value == NULL)
 			break;
@@ -254,12 +261,6 @@ struct _xneur_handle *xneur_handle_create (void)
 		if (lang_dir != NULL)
 			free(lang_dir);
 
-		if (handle->languages[lang].dictionary->data_count == 0 &&
-		    handle->languages[lang].proto->data_count == 0 &&
-		    handle->languages[lang].big_proto->data_count == 0)
-		{
-			handle->languages[lang].excluded	= TRUE;
-		}
 	}
 
 #ifdef WITH_ASPELL
@@ -347,7 +348,22 @@ struct _xneur_handle *xneur_handle_create (void)
 		}
 	}
 #endif
-	
+
+	for (int lang = 0; lang < handle->total_languages; lang++)
+	{
+		handle->languages[lang].disable_auto_detection |=
+			handle->languages[lang].excluded || (
+				handle->languages[lang].dictionary->data_count == 0 &&
+				handle->languages[lang].proto->data_count == 0 &&
+				handle->languages[lang].big_proto->data_count == 0
+#ifdef WITH_ASPELL
+				&& handle->has_spell_checker[lang] == 0
+#endif
+#ifdef WITH_ENCHANT
+				&& handle->has_enchant_checker[lang] == 0
+#endif
+			);
+	}
 	return handle;
 }
 
@@ -364,7 +380,7 @@ void xneur_handle_destroy (struct _xneur_handle *handle)
 #endif
 		
 #ifdef WITH_ENCHANT
-		if (handle->enchant_dicts[lang])
+		if ((handle->enchant_dicts[lang] != NULL) && (handle->has_enchant_checker[lang]))
 			enchant_broker_free_dict (handle->enchant_broker, handle->enchant_dicts[lang]);
 #endif
 		
@@ -382,9 +398,11 @@ void xneur_handle_destroy (struct _xneur_handle *handle)
 
 		if (handle->languages[lang].pattern != NULL)
 			handle->languages[lang].pattern->uninit(handle->languages[lang].pattern);
-		
-		free(handle->languages[lang].name);
-		free(handle->languages[lang].dir);
+
+		if (handle->languages[lang].name != NULL)
+			free(handle->languages[lang].name);
+		if (handle->languages[lang].dir != NULL)
+			free(handle->languages[lang].dir);
 	}
 	handle->total_languages = 0;
 	if (handle->languages != NULL)
@@ -409,12 +427,15 @@ int xneur_get_layout (struct _xneur_handle *handle, char *word)
 {
 	if (!word || handle == NULL)
 		return -1;
-	
-	struct _buffer *buffer = buffer_init(handle);
+
+	struct _buffer *buffer = buffer_init(handle, main_window->keymap);
+
 	buffer->set_content(buffer, word);
 	int cur_lang = get_curr_keyboard_group();
 	int new_lang = check_lang(handle, buffer, cur_lang);
+
 	buffer->uninit(buffer);
+
 	// The word is suitable for all languages, return -1
 	if (new_lang == NO_LANGUAGE)
 		new_lang = -1;
@@ -426,18 +447,23 @@ char *xneur_get_word (struct _xneur_handle *handle, char *word)
 {
 	if (!word || handle == NULL)
 		return NULL;
-	
-	struct _buffer *buffer = buffer_init(handle);
+
+	char *result = NULL;
+
+	struct _buffer *buffer = buffer_init(handle, main_window->keymap);
+
 	buffer->set_content(buffer, word);
 	int cur_lang = get_curr_keyboard_group();
 	int new_lang = check_lang(handle, buffer, cur_lang);
 	if (new_lang == NO_LANGUAGE)
-		return strdup(word);
+		result = strdup(word);
+	else
+		buffer->set_lang_mask(buffer, new_lang),
+		result = buffer->get_utf_string(buffer);
 
-	buffer->set_lang_mask(buffer, new_lang);
-	char *new_word = strdup(buffer->get_utf_string(buffer));
 	buffer->uninit(buffer);
 
-	return new_word;
+	return result;
 }
 
+
