@@ -245,7 +245,7 @@ static void buffer_save(struct _buffer *p, char *file_name, Window window)
 			continue;
 		}
 
-		char *symbol = keycode_to_symbol(p->keycode[i], -1, p->keycode_modifiers[i]);
+		char *symbol = p->keymap->keycode_to_symbol(p->keymap, p->keycode[i], -1, p->keycode_modifiers[i]);
 		if (symbol == NULL)
 		{
 			fprintf(stream, "<?>");
@@ -306,12 +306,12 @@ static void buffer_set_i18n_content(struct _buffer *p)
 		for (int i = 0; i < p->handle->total_languages; i++)
 		{
 		
-			char *symbol = keycode_to_symbol(p->keycode[k], i, modifier & (~ShiftMask));
+			char *symbol = p->keymap->keycode_to_symbol(p->keymap, p->keycode[k], i, modifier & (~ShiftMask));
 			if (symbol == NULL)
 				continue;
 			
-			char *symbol_unchanged = keycode_to_symbol(p->keycode[k], i, modifier);
-			if (symbol == NULL)
+			char *symbol_unchanged = p->keymap->keycode_to_symbol(p->keymap, p->keycode[k], i, modifier);
+			if (symbol_unchanged == NULL)
 				continue;
 			
 			p->i18n_content[i].content = (char *) realloc(p->i18n_content[i].content, (strlen(p->i18n_content[i].content) + strlen(symbol) + 1) * sizeof(char));
@@ -356,9 +356,7 @@ static void buffer_set_content(struct _buffer *p, const char *new_content)
 	memcpy(p->content, content, p->cur_pos);
 	free(content);
 
-	struct _keymap *keymap = keymap_init(p->handle);
-	keymap->convert_text_to_ascii(keymap, p->content, p->keycode, p->keycode_modifiers);
-	keymap->uninit(keymap);
+	p->keymap->convert_text_to_ascii(p->keymap, p->content, p->keycode, p->keycode_modifiers);
 
 	p->cur_pos = strlen(p->content);
 	set_new_size(p, p->cur_pos + 1);
@@ -371,13 +369,42 @@ static void buffer_set_content(struct _buffer *p, const char *new_content)
 
 static void buffer_change_case(struct _buffer *p)
 {
+	char *symbol = (char *) malloc((256 + 1) * sizeof(char));
+	
+	Display *display = XOpenDisplay(NULL);
+	XEvent event;
+	event.type		= KeyPress;
+	event.xkey.type		= KeyPress;
+	event.xkey.root		= RootWindow(display, DefaultScreen(display));
+	event.xkey.subwindow	= None;
+	event.xkey.same_screen	= True;
+	event.xkey.display	= display;
+	event.xkey.state	= 0;
+	event.xkey.keycode	= XKeysymToKeycode(display, XK_space);
+	event.xkey.time		= CurrentTime;
+	
 	for (int i = 0; i < p->cur_pos; i++)
 	{
+		event.xkey.keycode	= p->keycode[i];
+		event.xkey.state	= p->keycode_modifiers[i];
+
+		int nbytes = XLookupString((XKeyEvent *) &event, symbol, 256, NULL, NULL);
+		if (nbytes <= 0)
+			continue;
+
+		symbol[nbytes] = NULLSYM;	
+
+		if (ispunct(symbol[0]) || isdigit(symbol[0]))
+			continue;
+
 		if (p->keycode_modifiers[i] & ShiftMask)
 			p->keycode_modifiers[i] = (p->keycode_modifiers[i] & ~ShiftMask);
 		else
 			p->keycode_modifiers[i] = (p->keycode_modifiers[i] | ShiftMask);
 	}
+	
+	free(symbol);
+	XCloseDisplay(display);
 }
 
 static void buffer_rotate_layout(struct _buffer *p)
@@ -422,11 +449,11 @@ static void buffer_add_symbol(struct _buffer *p, char sym, KeyCode keycode, int
 
 	for (int i = 0; i < p->handle->total_languages; i++)
 	{
-		char *symbol = keycode_to_symbol(keycode, i, modifier & (~ShiftMask));
+		char *symbol = p->keymap->keycode_to_symbol(p->keymap, keycode, i, modifier & (~ShiftMask));
 		if (symbol == NULL)
 			continue;
 
-		char *symbol_unchanged = keycode_to_symbol(keycode, i, modifier);
+		char *symbol_unchanged = p->keymap->keycode_to_symbol(p->keymap, keycode, i, modifier);
 		if (symbol_unchanged == NULL)
 			continue;
 		
@@ -517,7 +544,7 @@ static char *buffer_get_utf_string_on_kbd_group(struct _buffer *p, int group)
 		{
 			state = state & (~keyboard_groups[j]);
 		}
-		char *symbol = keycode_to_symbol(p->keycode[i], group, state);
+		char *symbol = p->keymap->keycode_to_symbol(p->keymap, p->keycode[i], group, state);
 		if (symbol)
 		{
 			utf_string = (char *) realloc(utf_string, strlen(utf_string) * sizeof(char) + strlen(symbol) + 1);
@@ -573,12 +600,14 @@ static void buffer_uninit(struct _buffer *p)
 	log_message(DEBUG, _("String is freed"));
 }
 
-struct _buffer* buffer_init(struct _xneur_handle *handle)
+struct _buffer* buffer_init(struct _xneur_handle *handle, struct _keymap *keymap)
 {
 	struct _buffer *p = (struct _buffer *) malloc(sizeof(struct _buffer));
 	bzero(p, sizeof(struct _buffer));
 
 	p->handle = handle;
+
+	p->keymap = keymap;
 	
 	p->cur_size		= INIT_STRING_LENGTH;
 
