@@ -63,16 +63,33 @@ const char *verbose_focus_status[]	= {"Processed", "Changed Focus", "Unchanged F
 }*/
 
 // Private
-static int get_focus(struct _focus *p, int *forced_mode, int *focus_status, int *autocomplementation_mode)
+int focus_get_focused_window(struct _focus *p)
+{
+	if (p) {};
+	Window new_window;
+	int revert_to;
+	XGetInputFocus(main_window->display, &new_window, &revert_to);
+
+	return new_window;
+}
+
+static int get_focus(struct _focus *p, int *forced_mode, int *focus_status, int *autocompletion_mode)
 {
 	*forced_mode	= FORCE_MODE_NORMAL;
 	*focus_status	= FOCUS_NONE;
-	*autocomplementation_mode	= AUTOCOMPLEMENTATION_INCLUDED;
-	
+	*autocompletion_mode	= AUTOCOMPLETION_INCLUDED;
+
+	char *new_app_name = NULL;
+		
+	// Clear masking on unfocused window
+	p->update_grab_events(p, LISTEN_DONTGRAB_INPUT);
+	p->update_events(p, LISTEN_DONTGRAB_INPUT);
+
 	Window new_window;
 	int show_message = TRUE;
 	while (TRUE)
 	{
+		usleep(500000);
 		// This code commented be cause function XGrabKey for _NET_ACTIVE_WINDOW 
 		// dont process modifier keys (see utils.h)
 		/*if (main_window->_NET_SUPPORTED)
@@ -99,9 +116,13 @@ static int get_focus(struct _focus *p, int *forced_mode, int *focus_status, int
 		//}
 
 		// Catch not empty and not system window
-		if (new_window != None/* && new_window > 1000*/)
-			break;
-
+		if (new_window != None && new_window > 1000)
+		{
+			new_app_name = get_wm_class_name(new_window);
+			if (new_app_name != NULL)
+				break;
+		}
+		
 		if (show_message)
 		{
 			log_message(DEBUG, _("New window empty"));
@@ -110,28 +131,43 @@ static int get_focus(struct _focus *p, int *forced_mode, int *focus_status, int
 		usleep(1000);
 	}
 
-	char *new_app_name = get_wm_class_name(new_window);
-	if (new_app_name != NULL)
-	{
+	//char *new_app_name = get_wm_class_name(new_window);
+	//if (new_app_name != NULL)
+	//{
 		if (xconfig->excluded_apps->exist(xconfig->excluded_apps, new_app_name, BY_PLAIN))
 			*focus_status = FOCUS_EXCLUDED;
-
+		
 		if (xconfig->auto_apps->exist(xconfig->auto_apps, new_app_name, BY_PLAIN))
 			*forced_mode = FORCE_MODE_AUTO;
 		else if (xconfig->manual_apps->exist(xconfig->manual_apps, new_app_name, BY_PLAIN))
 			*forced_mode = FORCE_MODE_MANUAL;
 
-		if (xconfig->autocomplementation_excluded_apps->exist(xconfig->autocomplementation_excluded_apps, new_app_name, BY_PLAIN))
-			*autocomplementation_mode	= AUTOCOMPLEMENTATION_EXCLUDED;
-	}
-	else
-		*focus_status = FOCUS_EXCLUDED;
+		if (xconfig->autocompletion_excluded_apps->exist(xconfig->autocompletion_excluded_apps, new_app_name, BY_PLAIN))
+			*autocompletion_mode	= AUTOCOMPLETION_EXCLUDED;
+	//}
+	//else
+	//	*focus_status = FOCUS_EXCLUDED;
 
 	Window old_window = p->owner_window;
 	if (new_window == old_window)
 	{
 		if (new_app_name != NULL)
 			free(new_app_name);
+		if (xconfig->troubleshoot_full_screen)
+		{
+			Window root_return;
+			int x_return, y_return, root_x_return, root_y_return;
+			unsigned int width_return, height_return, root_width_return, root_height_return;
+			unsigned int border_width_return;
+			unsigned int depth_return;
+			XGetGeometry(main_window->display, p->parent_window, &root_return, &x_return, &y_return, &width_return, 
+							&height_return, &border_width_return, &depth_return);
+			XGetGeometry(main_window->display, root_return, &root_return, &root_x_return, &root_y_return, &root_width_return, 
+							&root_height_return, &border_width_return, &depth_return);
+			if ((x_return == 0) && (y_return == 0) && 
+			    (width_return == root_width_return) && (height_return == root_height_return))
+				*forced_mode = FORCE_MODE_MANUAL;
+		}
 		return FOCUS_UNCHANGED;
 	}
 
@@ -154,55 +190,104 @@ static int get_focus(struct _focus *p, int *forced_mode, int *focus_status, int
 		p->parent_window = parent_window;
 	}
 
-	// Clear masking on unfocused window
-	p->update_events(p, LISTEN_DONTGRAB_INPUT);
 	// Replace unfocused window to focused window
 	p->owner_window = new_window;
 
-	log_message(DEBUG, _("Process new window (ID %d) with name '%s' (status %s, mode %s)"), new_window, new_app_name, verbose_focus_status[*focus_status], verbose_forced_mode[*forced_mode]);
-
+	if (xconfig->troubleshoot_full_screen)
+	{
+		Window root_return;
+		int x_return, y_return, root_x_return, root_y_return;
+		unsigned int width_return, height_return, root_width_return, root_height_return;
+		unsigned int border_width_return;
+		unsigned int depth_return;
+		XGetGeometry(main_window->display, p->parent_window, &root_return, &x_return, &y_return, &width_return, 
+						&height_return, &border_width_return, &depth_return);
+		XGetGeometry(main_window->display, root_return, &root_return, &root_x_return, &root_y_return, &root_width_return, 
+						&root_height_return, &border_width_return, &depth_return);
+		if ((x_return == 0) && (y_return == 0) && 
+			(width_return == root_width_return) && (height_return == root_height_return))
+			*forced_mode = FORCE_MODE_MANUAL;
+	}
+	
+	log_message(DEBUG, _("Process new window (ID %d) with name '%s' (status %s, mode %s)"), new_window, new_app_name, _(verbose_focus_status[*focus_status]), _(verbose_forced_mode[*forced_mode]));
+	
 	if (new_app_name != NULL)
 		free(new_app_name);
 	return FOCUS_CHANGED;
 }
 
-static int focus_get_focus_status(struct _focus *p, int *forced_mode, int *focus_status, int *autocomplementation_mode)
+static int focus_get_focus_status(struct _focus *p, int *forced_mode, int *focus_status, int *autocompletion_mode)
 {
-	int focus = get_focus(p, forced_mode, focus_status, autocomplementation_mode);
-
-	if (focus == FOCUS_UNCHANGED)
-		return p->last_focus;
-
-	if (focus == FOCUS_CHANGED)
-		p->last_focus = FOCUS_UNCHANGED;
-	else
-		p->last_focus = focus;
-
+	int focus = get_focus(p, forced_mode, focus_status, autocompletion_mode);
+	
+	p->last_focus = *focus_status;
+	if (!xconfig->tracking_input)
+		p->last_focus = FOCUS_EXCLUDED;
+	
 	return focus;
 }
 
-static void focus_update_events(struct _focus *p, int mode)
+static void focus_update_grab_events(struct _focus *p, int mode)
 {
+	char *owner_window_name = get_wm_class_name(p->owner_window);
 	if (mode == LISTEN_DONTGRAB_INPUT)
 	{
+		log_message (DEBUG, _("Interception of events in the window (ID %d) with name '%s' OFF"), p->owner_window, owner_window_name);
+		
 		// Event unmasking
 		grab_button(FALSE);
 		
 		// Ungrabbing special key (Enter, Tab and other)
 		grab_spec_keys(p->owner_window, FALSE);
 
-		//set_mask_to_window(rw, FOCUS_CHANGE_MASK);
-		set_event_mask(p->owner_window, FOCUS_CHANGE_MASK);
+		set_mask_to_window(p->owner_window, FOCUS_CHANGE_MASK);
+		//set_event_mask(p->owner_window, None);
 	}
 	else
 	{
+		log_message (DEBUG, _("Interception of events in the window (ID %d) with name '%s' ON"), p->owner_window, owner_window_name);
+		
 		// Event masking
-		grab_button(TRUE);
 		
 		// Grabbing special key (Enter, Tab and other)
-		grab_spec_keys(p->owner_window, TRUE);
+		if (p->last_focus != FOCUS_EXCLUDED)
+		{
+			if (xconfig->tracking_mouse)
+			  grab_button(TRUE);
+			grab_spec_keys(p->owner_window, TRUE);
+			set_event_mask(p->owner_window, INPUT_HANDLE_MASK | FOCUS_CHANGE_MASK | EVENT_KEY_MASK);
+		}
+		else
+		{
+			grab_button(FALSE);
+			grab_spec_keys(p->owner_window, FALSE);
+			set_event_mask(p->owner_window, FOCUS_CHANGE_MASK);
+		}
+	}
 
-		set_event_mask(p->owner_window, INPUT_HANDLE_MASK | FOCUS_CHANGE_MASK | EVENT_KEY_MASK);
+	p->last_parent_window = p->parent_window;
+	
+	if (owner_window_name != NULL)
+		free(owner_window_name);
+}
+
+static void focus_update_events(struct _focus *p, int mode)
+{
+	
+	if (mode == LISTEN_DONTGRAB_INPUT)
+	{
+		set_event_mask(p->owner_window, None);
+	}
+	else
+	{
+		if (p->last_focus != FOCUS_EXCLUDED)
+		{
+			set_event_mask(p->owner_window, INPUT_HANDLE_MASK | FOCUS_CHANGE_MASK | EVENT_KEY_MASK);
+		}
+		else
+		{
+			set_event_mask(p->owner_window, FOCUS_CHANGE_MASK);
+		}
 	}
 
 	p->last_parent_window = p->parent_window;
@@ -222,6 +307,8 @@ struct _focus* focus_init(void)
 
 	// Functions mapping
 	p->get_focus_status	= focus_get_focus_status;
+	p->get_focused_window = focus_get_focused_window;
+	p->update_grab_events	= focus_update_grab_events;
 	p->update_events	= focus_update_events;
 	p->uninit		= focus_uninit;
 
