@@ -38,6 +38,22 @@
 
 #include "keymap.h"
 
+#define keycode_to_symbol_cache_size 64
+#define symbol_to_keycode_cache_size 64
+
+struct symbol_to_keycode_pair 
+{
+	char* symbol; size_t symbol_size; int preferred_lang;
+	KeyCode kc; int modifier; char ascii; int preferred_lang_result;
+};
+
+struct keycode_to_symbol_pair 
+{
+	KeyCode kc; int group; int state;
+	char* symbol; size_t symbol_size;
+};
+
+
 #define NumlockMask 0x10
 
 static const int keyboard_groups[]	= {0x00000000, 0x00002000, 0x00004000, 0x00006000};
@@ -67,16 +83,15 @@ int get_languages_mask(void)
 	return languages_mask;
 }
 
-char* keycode_to_symbol(KeyCode kc, int group, int state)
+static char* keymap_keycode_to_symbol_real(struct _keymap *p, KeyCode kc, int group, int state)
 {
-	Display *display = XOpenDisplay(NULL);
 	XEvent event;
 	event.type		= KeyPress;
 	event.xkey.type		= KeyPress;
-	event.xkey.root		= RootWindow(display, DefaultScreen(display));
+	event.xkey.root		= RootWindow(p->display, DefaultScreen(p->display));
 	event.xkey.subwindow	= None;
 	event.xkey.same_screen	= True;
-	event.xkey.display	= display;
+	event.xkey.display	= p->display;
 	event.xkey.keycode = kc;
 	event.xkey.state = 0;
 	event.xkey.time		= CurrentTime;
@@ -88,7 +103,6 @@ char* keycode_to_symbol(KeyCode kc, int group, int state)
 	char *symbol = (char *) malloc((256 + 1) * sizeof(char));
 
 	int nbytes = XLookupString((XKeyEvent *) &event, symbol, 256, NULL, NULL);
-	XCloseDisplay(display);
 		
 	if (nbytes <= 0)
 	{
@@ -117,11 +131,9 @@ char* keycode_to_symbol(KeyCode kc, int group, int state)
 		{
 			if (setlocale(LC_ALL, locales->data[i].string) != NULL)
 			{
-				Display *dpy = XOpenDisplay(NULL);
-				event.xkey.root		= RootWindow(dpy, DefaultScreen(dpy));
-				event.xkey.display  = dpy;
+				event.xkey.root		= RootWindow(p->display, DefaultScreen(p->display));
+				event.xkey.display  = p->display;
 				nbytes = XLookupString((XKeyEvent *) &event, symbol, 256, NULL, NULL);
-				XCloseDisplay(dpy);
 				
 				setlocale(LC_ALL, ""); 
 				
@@ -134,8 +146,8 @@ char* keycode_to_symbol(KeyCode kc, int group, int state)
 			}
 		}
 
-		log_message(ERROR, _("Not find symbol for keycode %d and modifier 0x%x!"), event.xkey.keycode, event.xkey.state);
-		log_message(ERROR, _("Try run the programm with command \"env LC_ALL=<LOCALE> %s\", \nwhere LOCALE available over command \"locale -a\""), PACKAGE);
+		log_message(ERROR, _("Failed to look up symbol for keycode %d and modifier 0x%x!"), event.xkey.keycode, event.xkey.state);
+		log_message(ERROR, _("Try run the program with command \"env LC_ALL=<LOCALE> %s\", \nwhere LOCALE available over command \"locale -a\""), PACKAGE);
 		symbol[0] = NULLSYM;
 		strcat(symbol, " ");
 
@@ -148,12 +160,50 @@ char* keycode_to_symbol(KeyCode kc, int group, int state)
 	return symbol;
 }
 
+static char* keymap_keycode_to_symbol(struct _keymap *p, KeyCode kc, int group, int state)
+{
+	char *symbol;
+	struct keycode_to_symbol_pair *pr = NULL;
+
+	/* Look up cache. */
+	for (int i = 0; i < keycode_to_symbol_cache_size; i++) {
+		pr = p->keycode_to_symbol_cache + i;
+		if (pr->symbol && pr->kc == kc && pr->group == group && pr->state == state)
+			goto ret;
+	}
+
+	/* Miss. */
+
+	symbol = keymap_keycode_to_symbol_real(p, kc, group, state);
+	if (!symbol)
+		return symbol;
+
+	/* Just use next cache entry. LRU makes no sense here. */
+	p->keycode_to_symbol_cache_pos = (p->keycode_to_symbol_cache_pos + 1) % keycode_to_symbol_cache_size;
+
+	pr = p->keycode_to_symbol_cache + p->keycode_to_symbol_cache_pos;
+
+	pr->symbol_size = (strlen(symbol) + 1) * sizeof(char);
+	if (pr->symbol)
+		free(pr->symbol);
+	pr->symbol = symbol;
+	pr->kc     = kc;
+	pr->group  = group;
+	pr->state  = state;
+
+	ret:
+
+	symbol = (char *) malloc(pr->symbol_size);
+	memcpy(symbol, pr->symbol, pr->symbol_size);
+	return symbol;
+}
+
 int get_keycode_mod(int group)
 {
 	return keyboard_groups[group];
 }
 
-void get_keysyms_by_string(char *keyname, KeySym *lower, KeySym *upper)
+static void keymap_get_keysyms_by_string(struct _keymap *p, char *keyname, KeySym *lower, KeySym *upper)
 {
 	if (keyname == NULL)
 	{
@@ -164,49 +214,38 @@ void get_keysyms_by_string(char *keyname, KeySym *lower, KeySym *upper)
 
 	KeySym inbound_key = XStringToKeysym(keyname);
 
-	Display *display = XOpenDisplay(NULL);
-	
-	int min_keycode, max_keycode;
-	XDisplayKeycodes(display, &min_keycode, &max_keycode);
+	KeySym *keymap = p->keymap;
 
-	int keysyms_per_keycode;
-	KeySym *keymap = XGetKeyboardMapping(display, min_keycode, max_keycode - min_keycode + 1, &keysyms_per_keycode);
-	KeySym *to_free = keymap;
-	
-	for (int i = min_keycode; i <= max_keycode; i++)
+	for (int i = p->min_keycode; i <= p->max_keycode; i++)
 	{
 		for (int j = 0; j < 2; j++)
 		{
-			if (keymap[j] == NoSymbol)
+			if (p->keymap[j] == NoSymbol)
 				continue;
 
-			if (keymap[j] != inbound_key)
+			if (p->keymap[j] != inbound_key)
 				continue;
 
-			*lower = keymap[0];
-			*upper = keymap[1];
+			*lower = p->keymap[0];
+			*upper = p->keymap[1];
 
-			XFree(to_free);
-			XCloseDisplay(display);
+			p->keymap = keymap;
 			return;
 		}
 
-		keymap += keysyms_per_keycode;
+		p->keymap += p->keysyms_per_keycode;
 	}
 
-	XFree(to_free);
-	XCloseDisplay(display);
+	p->keymap = keymap;
 }
 
 // Private
 static int init_keymaps(struct _keymap *p)
 {
 	// Define all key codes and key symbols
-	Display *display = XOpenDisplay(NULL);
-	XDisplayKeycodes(display, &(p->min_keycode), &(p->max_keycode));
-	p->keymap = XGetKeyboardMapping(display, p->min_keycode, p->max_keycode - p->min_keycode + 1, &(p->keysyms_per_keycode));
-	XCloseDisplay(display);
-
+	XDisplayKeycodes(p->display, &(p->min_keycode), &(p->max_keycode));
+	p->keymap = XGetKeyboardMapping(p->display, p->min_keycode, p->max_keycode - p->min_keycode + 1, &(p->keysyms_per_keycode));
+	
 	if (!p->keymap)
 	{
 		log_message(ERROR, _("Unable to get keyboard mapping table"));
@@ -215,75 +254,42 @@ static int init_keymaps(struct _keymap *p)
 	return TRUE;
 }
 
-static void keymap_char_to_keycode(struct _keymap *p, char ch, KeyCode *kc, int *modifier)
+static char keymap_get_ascii_real(struct _keymap *p, const char *sym, int* preferred_lang, KeyCode *kc, int *modifier, size_t* symbol_len)
 {
-	Display *display = XOpenDisplay(NULL);
-	
-	if (ch == 10 || ch == 13)
+	if (*sym == 10 || *sym == 13)
 	{
-		*kc		= XKeysymToKeycode(display, XK_Return);
+		*kc		= XKeysymToKeycode(p->display, XK_Return);
 		*modifier	= 0;
-		XCloseDisplay(display);
-		return;
+		if (symbol_len)
+			*symbol_len = 1;
+		return *sym;
 	}
 
 	XEvent event;
 	event.type		= KeyPress;
 	event.xkey.type		= KeyPress;
-	event.xkey.root		= RootWindow(display, DefaultScreen(display));
+	event.xkey.root		= RootWindow(p->display, DefaultScreen(p->display));
 	event.xkey.subwindow	= None;
 	event.xkey.same_screen	= True;
-	event.xkey.display	= display;
+	event.xkey.display	= p->display;
 	event.xkey.state	= 0;
-	event.xkey.keycode	= XKeysymToKeycode(display, XK_space);
-	event.xkey.time		= CurrentTime;
-
-	char *symbol = (char *) malloc((256 + 1) * sizeof(char));
-
-	for (int i = p->min_keycode + 1; i <= p->max_keycode; i++)
-	{
-		event.xkey.keycode	= i;
-		event.xkey.state	= 0;
-
-		int nbytes = XLookupString((XKeyEvent *) &event, symbol, 256, NULL, NULL);
-		if ((nbytes > 0) && (symbol[0] == ch))
-			break;
-
-		event.xkey.state	= ShiftMask;
-
-		nbytes = XLookupString((XKeyEvent *) &event, symbol, 256, NULL, NULL);
-		if ((nbytes > 0) && (symbol[0] == ch))
-			break;
-	}
-
-	*kc		= event.xkey.keycode;
-	*modifier	= (event.xkey.state == ShiftMask) ? 1 : 0;
-	free(symbol);
-	
-	XCloseDisplay(display);
-}
-
-static char keymap_get_ascii(struct _keymap *p, const char *sym, KeyCode *kc, int *modifier)
-{
-	Display *display = XOpenDisplay(NULL);
-	XEvent event;
-	event.type		= KeyPress;
-	event.xkey.type		= KeyPress;
-	event.xkey.root		= RootWindow(display, DefaultScreen(display));
-	event.xkey.subwindow	= None;
-	event.xkey.same_screen	= True;
-	event.xkey.display	= display;
-	event.xkey.state	= 0;
-	event.xkey.keycode	= XKeysymToKeycode(display, XK_space);
+	event.xkey.keycode	= XKeysymToKeycode(p->display, XK_space);
 	event.xkey.time		= CurrentTime;
 	
 	char *symbol		= (char *) malloc((256 + 1) * sizeof(char));
 	char *prev_symbols	= (char *) malloc((256 + 1) * sizeof(char));
 
-	for (int lang = 0; lang < p->handle->total_languages; lang++)
+	int _preferred_lang = 0;
+	if (preferred_lang)
+		_preferred_lang = *preferred_lang;
+
+	for (int _lang = 0; _lang < p->handle->total_languages; _lang++)
 	{
-		if (lang == p->latin_group)
-			continue;
+		int lang = _lang;
+		if (lang == 0)
+			lang = _preferred_lang;
+		else if (lang <= _preferred_lang)
+			lang--;
 
 		KeySym *keymap = p->keymap;
 		for (int i = p->min_keycode; i <= p->max_keycode; i++)
@@ -321,6 +327,8 @@ static char keymap_get_ascii(struct _keymap *p, const char *sym, KeyCode *kc, in
 						if (strncmp(sym, symbol, strlen(symbol)) != 0)
 							continue;
 
+						size_t _symbol_len = strlen(symbol);
+
 						event.xkey.state = 0;
 						event.xkey.state |= state_masks[m];
 						event.xkey.state |= state_masks[n];
@@ -335,7 +343,10 @@ static char keymap_get_ascii(struct _keymap *p, const char *sym, KeyCode *kc, in
 						free(symbol);
 						*kc		= event.xkey.keycode;
 						*modifier	= get_keycode_mod(lang) | event.xkey.state;
-						XCloseDisplay(display);
+						if (symbol_len)
+							*symbol_len = _symbol_len;
+						if (preferred_lang)
+							*preferred_lang = lang;
 						return sym;
 					}
 				}
@@ -344,13 +355,64 @@ static char keymap_get_ascii(struct _keymap *p, const char *sym, KeyCode *kc, in
 		}
 	}
 
-	XCloseDisplay(display);
-	
 	free(prev_symbols);
 	free(symbol);
 	return NULLSYM;
 }
 
+static char keymap_get_ascii(struct _keymap *p, const char *sym, int* preferred_lang, KeyCode *kc, int *modifier, size_t* symbol_len)
+{
+	struct symbol_to_keycode_pair *pr = NULL;
+
+	int _preferred_lang = 0;
+	if (preferred_lang)
+		_preferred_lang = *preferred_lang;
+
+	size_t sym_size = strlen(sym);
+
+	/* Look up cache */
+
+	for (int i = 0; i < symbol_to_keycode_cache_size; i++) {
+		pr = p->symbol_to_keycode_cache + i;
+		if (pr->symbol &&
+		    pr->symbol_size <= sym_size &&
+		    pr->preferred_lang == _preferred_lang &&
+		    memcmp(pr->symbol, sym, pr->symbol_size) == 0)
+			goto ret;
+	}
+
+	/* Miss */
+
+	int preferred_lang_result = _preferred_lang;
+	char ascii = keymap_get_ascii_real(p, sym, &preferred_lang_result, kc, modifier, &sym_size);
+	if (!ascii)
+		return ascii;
+
+	p->symbol_to_keycode_cache_pos = (p->symbol_to_keycode_cache_pos + 1) % symbol_to_keycode_cache_size;
+
+	pr = p->symbol_to_keycode_cache + p->symbol_to_keycode_cache_pos;
+
+	pr->symbol = realloc(pr->symbol, sym_size + 1);
+	memcpy(pr->symbol, sym, sym_size);
+	pr->symbol[sym_size] = 0;
+	pr->symbol_size = sym_size;
+	pr->preferred_lang = _preferred_lang;
+	pr->kc = *kc;
+	pr->modifier = *modifier;
+	pr->preferred_lang_result = preferred_lang_result;
+	pr->ascii = ascii;
+
+	ret:
+
+	if (preferred_lang)
+		*preferred_lang = pr->preferred_lang_result;
+	if (symbol_len)
+		*symbol_len = pr->symbol_size;
+	*kc = pr->kc;
+	*modifier = pr->modifier;
+	return pr->ascii;
+}
+
 static char keymap_get_cur_ascii_char(struct _keymap *p, XEvent e)
 {
 	XKeyEvent *ke = (XKeyEvent *) &e;
@@ -383,27 +445,16 @@ static void keymap_convert_text_to_ascii(struct _keymap *p, char *text, KeyCode
 	int text_len = strlen(text);
 
 	int j = 0;
-	for (int i = 0; i < text_len; i++)
+	size_t symbol_len = 0;
+	int preferred_lang = 0;
+	for (int i = 0; i < text_len; i += symbol_len)
 	{
-		if (isascii(text[i]) || isspace(text[i]))
-		{
-			p->char_to_keycode(p, text[i], &kc[j], &kc_mod[j]);
-			text[j++] = text[i];
-			continue;
-		}
+		char new_symbol = p->get_ascii(p, &text[i], &preferred_lang, &kc[j], &kc_mod[j], &symbol_len);
 
-		char new_symbol = p->get_ascii(p, &text[i], &kc[j], &kc_mod[j]);
-
-		for(; i < text_len - 1; i++)
-		{
-			if (isascii(text[i + 1]) || isspace(text[i + 1]))
-				break;
-
-			if (p->get_ascii(p, &text[i + 1], &kc[i + 1], &kc_mod[i + 1]) != NULLSYM)
-				break;
-		}
-
-		text[j++] = new_symbol;
+		if (new_symbol != NULLSYM && symbol_len > 0)
+			text[j++] = new_symbol;
+		else
+			symbol_len = 1;
 	}
 
 	text[j] = NULLSYM;
@@ -420,8 +471,7 @@ static char* keymap_lower_by_keymaps(struct _keymap *p, int gr, char *text)
 
 	char *newtext = strdup(text);
 
-	Display *display = XOpenDisplay(NULL);
-	
+
 	KeySym *keymap = p->keymap;
 	for (int i = p->min_keycode; i <= p->max_keycode; i++)
 	{
@@ -447,10 +497,10 @@ static char* keymap_lower_by_keymaps(struct _keymap *p, int gr, char *text)
 					XEvent event;
 					event.type		= KeyPress;
 					event.xkey.type		= KeyPress;
-					event.xkey.root		= RootWindow(display, DefaultScreen(display));
+					event.xkey.root		= RootWindow(p->display, DefaultScreen(p->display));
 					event.xkey.subwindow	= None;
 					event.xkey.same_screen	= True;
-					event.xkey.display	= display;
+					event.xkey.display	= p->display;
 					event.xkey.keycode	= i;
 					event.xkey.state	= get_keycode_mod(gr);
 					event.xkey.state	|= state_masks[m];
@@ -492,24 +542,91 @@ static char* keymap_lower_by_keymaps(struct _keymap *p, int gr, char *text)
 	free(symbol_new);
 	free(symbol_old);
 
-	XCloseDisplay(display);
-	
 	return newtext;
 }
 
 static void keymap_uninit(struct _keymap *p)
 {
+	p->purge_caches(p);
+	
 	if (p->keymap != NULL)
 		XFree(p->keymap);
+
+	free(p->keycode_to_symbol_cache);
+	free(p->symbol_to_keycode_cache);
+
 	free(p);
 }
 
-struct _keymap* keymap_init(struct _xneur_handle *handle)
+static void get_offending_modifiers (struct _keymap *p)
+{
+	p->numlock_mask = 0;
+	p->scrolllock_mask = 0;
+	p->capslock_mask = 0;
+	
+	int i;
+	XModifierKeymap *modmap;
+	KeyCode nlock, slock;
+	static int mask_table[8] = {
+								ShiftMask, LockMask, ControlMask, Mod1Mask,
+								Mod2Mask, Mod3Mask, Mod4Mask, Mod5Mask
+								};
+
+	nlock = XKeysymToKeycode (p->display, XK_Num_Lock);
+	slock = XKeysymToKeycode (p->display, XK_Scroll_Lock);
+
+	/*
+	* Find out the masks for the NumLock and ScrollLock modifiers,
+	* so that we can bind the grabs for when they are enabled too.
+	*/
+	modmap = XGetModifierMapping (p->display);
+
+	if (modmap != NULL && modmap->max_keypermod > 0)
+	{
+		for (i = 0; i < 8 * modmap->max_keypermod; i++)
+		{
+			if (modmap->modifiermap[i] == nlock && nlock != 0)
+				p->numlock_mask = mask_table[i / modmap->max_keypermod];
+			else if (modmap->modifiermap[i] == slock && slock != 0)
+				p->scrolllock_mask = mask_table[i / modmap->max_keypermod];
+		}
+	}
+
+	p->capslock_mask = LockMask;
+
+	if (modmap)
+		XFreeModifiermap (modmap);
+}
+
+static void keymap_purge_caches(struct _keymap *p)
+{
+	for (int i = 0; i < keycode_to_symbol_cache_size; i++) 
+	{
+		struct keycode_to_symbol_pair* pr = p->keycode_to_symbol_cache + i;
+		if (pr->symbol)
+			free(pr->symbol),
+			pr->symbol = NULL,
+			pr->symbol_size = 0;
+	}
+
+	for (int i = 0; i < symbol_to_keycode_cache_size; i++) 
+	{
+		struct symbol_to_keycode_pair* pr = p->symbol_to_keycode_cache + i;
+		if (pr->symbol)
+			free(pr->symbol),
+			pr->symbol = NULL,
+			pr->symbol_size = 0;
+	}
+
+}
+
+struct _keymap* keymap_init(struct _xneur_handle *handle, Display *display)
 {
 	struct _keymap *p = (struct _keymap *) malloc(sizeof(struct _keymap));
 	bzero(p, sizeof(struct _keymap));
 
 	p->handle = handle;
+	p->display = display;
 	
 	if (!locale_create() || !init_keymaps(p))
 	{
@@ -517,12 +634,23 @@ struct _keymap* keymap_init(struct _xneur_handle *handle)
 		return NULL;
 	}
 
+	p->keycode_to_symbol_cache = (struct keycode_to_symbol_pair *)calloc(keycode_to_symbol_cache_size, sizeof(struct keycode_to_symbol_pair));
+	p->symbol_to_keycode_cache = (struct symbol_to_keycode_pair *)calloc(symbol_to_keycode_cache_size, sizeof(struct symbol_to_keycode_pair));
+	p->keycode_to_symbol_cache_pos = 0;
+	p->symbol_to_keycode_cache_pos = 0;
+
+	get_offending_modifiers(p);
+
+	p->purge_caches			= keymap_purge_caches;
+	p->get_keysyms_by_string	= keymap_get_keysyms_by_string;
+	p->keycode_to_symbol		= keymap_keycode_to_symbol;
 	p->get_ascii			= keymap_get_ascii;
 	p->get_cur_ascii_char		= keymap_get_cur_ascii_char;
 	p->convert_text_to_ascii	= keymap_convert_text_to_ascii;
-	p->char_to_keycode		= keymap_char_to_keycode;
 	p->lower_by_keymaps		= keymap_lower_by_keymaps;
 	p->uninit			= keymap_uninit;
 
 	return p;
 }
+
+
