@@ -33,20 +33,18 @@ struct _program
 	int  changed_manual;
 	int  app_forced_mode;
 	int  app_focus_mode;
-	int  app_autocomplementation_mode;
+	int  app_autocompletion_mode;
 	
 	int  action_mode;
 
 	int  last_layout;
-	int  last_window;
-	
-	KeySym prev_key;
-	int prev_key_mod;
-	struct _list_char *modifiers_stack;
+	Window  last_window;
+
+	int user_action;
+	enum _hotkey_action manual_action;
 	
 	void (*layout_update) (struct _program *p);
 	void (*update) (struct _program *p);
-	void (*update_modifiers_stack) (struct _program *p);
 	void (*on_key_action) (struct _program *p, int type);
 	void (*process_input) (struct _program *p);
 	int  (*perform_manual_action) (struct _program *p, enum _hotkey_action action);
@@ -60,6 +58,11 @@ struct _program
 	void (*check_space_with_bracket) (struct _program *p);
 	void (*check_brackets_with_symbols) (struct _program *p);
 	void (*check_capital_letter_after_dot) (struct _program *p);
+	void (*check_two_space) (struct _program *p);
+	void (*check_two_minus) (struct _program *p);
+	void (*check_copyright) (struct _program *p);
+	void (*check_trademark) (struct _program *p);
+	void (*check_registered) (struct _program *p);
 	void (*check_pattern) (struct _program *p, int selection);
 	void (*change_word) (struct _program *p, enum _change_action action);
 	void (*add_word_to_dict) (struct _program *p, int new_lang);
