@@ -22,6 +22,7 @@
 
 #include "window.h"
 #include "defines.h"
+#include "bind_table.h"
 #include "xnconfig.h"
 
 #include "types.h"
@@ -128,7 +129,7 @@ void grab_button(int is_grab)
 	int status;
 	if (is_grab)
 	{
-		status = XGrabButton(main_window->display, Button1, AnyModifier, rw, TRUE, ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync, None, None);
+		status = XGrabButton(main_window->display, Button1, AnyModifier, rw, FALSE, ButtonPressMask|ButtonReleaseMask, GrabModeSync, GrabModeAsync, None, None);
 		XSync (main_window->display, FALSE);
 	}
 	else
@@ -156,36 +157,64 @@ void grab_key(Window window, KeyCode kc, int is_grab)
 		log_message(ERROR, _("Failed to %s keyboard with error BadWindow"), grab_ungrab[is_grab]);
 }
 
-void grab_spec_keys(Window window, int is_grab)
+void grab_modifier_keys(Window window, int is_grab)
 {
-	int status;
-	if (is_grab)
+	int i, k = 0;
+
+	XModifierKeymap *modmap = XGetModifierMapping (main_window->display);
+		
+	for (i = 0; i < 8; i++) 
 	{
-		// Grab all keys...
-		status = XGrabKey(main_window->display, AnyKey, AnyModifier, window, TRUE, GrabModeAsync, GrabModeAsync);
-		// ...without ModKeys.
-		for (int i = 0; i < total_mod_keys; i++)
+		int j;
+
+		for (j = 0; j < modmap->max_keypermod; j++) 
 		{
-			KeyCode kc = XKeysymToKeycode(main_window->display, mod_keys[i]);
-			if (kc == 0)
-				continue;
-			grab_key(window, kc, FALSE);
+			if (modmap->modifiermap[k]) 
+			{
+				if (is_grab)
+					XGrabKey(main_window->display, modmap->modifiermap[k], AnyModifier, window, FALSE, GrabModeAsync, GrabModeAsync);
+				else
+					XUngrabKey(main_window->display, modmap->modifiermap[k], AnyModifier, window);	
+			}
+			k++;
 		}
 	}
-	else
-		status = XUngrabKeyboard(main_window->display, CurrentTime);
 
-	if (status == BadValue)
-		log_message(ERROR, _("Failed to %s keyboard with error BadValue"), grab_ungrab[is_grab]);
-	else if (status == BadWindow)
-		log_message(ERROR, _("Failed to %s keyboard with error BadWindow"), grab_ungrab[is_grab]);
+	if (modmap)
+		XFreeModifiermap (modmap);
+	
+	return;
+}
+	
+void grab_spec_keys(Window window, int is_grab)
+{
+	if (is_grab)
+	{
+		// Ungrab hotkeys in root window
+		XUngrabKey(main_window->display, AnyKey, AnyModifier, DefaultRootWindow (main_window->display));
+		// Grab all keys...
+		XGrabKey(main_window->display, AnyKey, AnyModifier, window, FALSE, GrabModeAsync, GrabModeAsync);
+		// ...without ModKeys.
+		grab_modifier_keys(window, FALSE);
+	}
+	else
+	{
+		// Ungrab all keys in app window...
+		XUngrabKey(main_window->display, AnyKey, AnyModifier, window);
+		// ... and with hotkeys
+		XUngrabKey(main_window->display, AnyKey, AnyModifier, DefaultRootWindow (main_window->display));
+		grab_manual_action();
+		grab_user_action();
+		// ...without ModKeys.
+		grab_modifier_keys(DefaultRootWindow (main_window->display), FALSE);
+	}
 }
 
 void grab_keyboard(Window window, int is_grab)
 {
 	int status;
 	if (is_grab)
-		status = XGrabKeyboard(main_window->display, window, TRUE, GrabModeAsync, GrabModeAsync, CurrentTime);
+		status = XGrabKeyboard(main_window->display, window, FALSE, GrabModeAsync, GrabModeAsync, CurrentTime);
 	else
 		status = XUngrabKeyboard(main_window->display, CurrentTime);
 
@@ -226,7 +255,6 @@ unsigned char *get_win_prop(Window window, Atom atom, long *nitems, Atom *type,
 	Atom actual_type;
 	int actual_format;
 	unsigned long _nitems;
-	unsigned long nbytes;
 	unsigned long bytes_after; /* unused */
 	unsigned char *prop;
 	int status;
@@ -238,15 +266,7 @@ unsigned char *get_win_prop(Window window, Atom atom, long *nitems, Atom *type,
 	if ((status == BadWindow) || (status != Success)) 
 		return NULL;
 
-	if (actual_format == 32)
-		nbytes = sizeof(long);
-	else if (actual_format == 16)
-		nbytes = sizeof(short);
-	else if (actual_format == 8)
-		nbytes = 1;
-	else if (actual_format == 0)
-		nbytes = 0;
-
+	
 	*nitems = _nitems;
 	*type = actual_type;
 	*size = actual_format;
