@@ -22,15 +22,19 @@
 #ifndef _TYPES_H_
 #define _TYPES_H_
 
+#ifndef TRUE
 #define TRUE		1
+#endif
+
+#ifndef FALSE
 #define FALSE		0
+#endif
 
 #define NULLSYM		0
 
-#define DICT_NAME			"dict"
+#define DICT_NAME			"dictionary"
 #define PROTO_NAME			"proto"
-#define BIG_PROTO_NAME			"proto3"
-#define REGEXP_NAME			"regexp"
+#define BIG_PROTO_NAME		"proto3"
 #define PATTERN_NAME		"pattern"
 
 #define LANGUAGEDIR			"languages"
