@@ -27,6 +27,10 @@
 
 #include <libnotify/notify.h>
 
+#ifndef NOTIFY_CHECK_VERSION /* macro did not exist before libnotify-0.5.2 */
+#  define NOTIFY_CHECK_VERSION(x,y,z) 0
+#endif
+
 #include <pthread.h>
 #include <stdio.h>
 #include <string.h>
@@ -42,10 +46,9 @@
 
 extern struct _xneur_config *xconfig;
 
-static const char *icon = "distributor-logo";
+static const char *icon = "xneur";
 static const char *type = NULL;
 static const NotifyUrgency urgency = NOTIFY_URGENCY_CRITICAL;
-static const int expire_timeout = NOTIFY_EXPIRES_DEFAULT;
 
 time_t timestamp = 0;
 
@@ -74,11 +77,15 @@ static void popup_show_thread(struct _popup_body *popup_body)
 		popup_body->content = NULL;
 	}
 	
-	NotifyNotification *notify = notify_notification_new(popup_body->header, popup_body->content, icon, NULL);
-
+#if NOTIFY_CHECK_VERSION(0,7,0)
+	NotifyNotification *notify = notify_notification_new(popup_body->header, popup_body->content, icon);
+#else
+ 	NotifyNotification *notify = notify_notification_new(popup_body->header, popup_body->content, icon, NULL);
+#endif
+		
 	notify_notification_set_category(notify, type);
 	notify_notification_set_urgency(notify, urgency);
-	notify_notification_set_timeout(notify, expire_timeout);
+	notify_notification_set_timeout(notify, xconfig->popup_expire_timeout);
 
 	notify_notification_show(notify, NULL);
 
@@ -90,11 +97,6 @@ static void popup_show_thread(struct _popup_body *popup_body)
 
 	notify_notification_clear_actions(notify);
 
-	GError *error = NULL;
-	notify_notification_close(notify, &error);
-	if (error != NULL)
-		g_error_free(error);
-	
 	g_object_unref(G_OBJECT(notify));
 }
 
