@@ -122,22 +122,17 @@ static void xneur_load_config(void)
 	log_message(LOG, _("Keyboard layouts present in system:"));
 	for (int lang = 0; lang < xconfig->handle->total_languages; lang++)
 	{
-		if ((xconfig->handle->languages[lang].dictionary->data_count == 0 &&
-		    xconfig->handle->languages[lang].proto->data_count == 0 &&
-		    xconfig->handle->languages[lang].big_proto->data_count == 0) || 
-			(xconfig->handle->languages[lang].excluded))
-		{
+		if (xconfig->handle->languages[lang].excluded)
 			log_message(LOG, _("   Excluded XKB Group '%s', layout '%s', group '%d'"), xconfig->handle->languages[lang].name, xconfig->handle->languages[lang].dir, lang);
-		}
 		else
-			log_message(LOG, _("   Encluded XKB Group '%s', layout '%s', group '%d'"), xconfig->handle->languages[lang].name, xconfig->handle->languages[lang].dir, lang);
+			log_message(LOG, _("   Included XKB Group '%s', layout '%s', group '%d'"), xconfig->handle->languages[lang].name, xconfig->handle->languages[lang].dir, lang);
 
 		char *lang_name = xconfig->handle->languages[lang].name;
 
 		log_message(DEBUG, _("      %s dictionary has %d records"), lang_name, xconfig->handle->languages[lang].dictionary->data_count);
 		log_message(DEBUG, _("      %s proto has %d records"), lang_name, xconfig->handle->languages[lang].proto->data_count);
 		log_message(DEBUG, _("      %s big proto has %d records"), lang_name, xconfig->handle->languages[lang].big_proto->data_count);
-#ifdef WITH_ASPELL		
+#ifdef WITH_ASPELL
 		if (xconfig->handle->has_spell_checker[lang])
 		{
 			log_message(DEBUG, _("      %s aspell dictionary loaded"), lang_name);
@@ -147,7 +142,7 @@ static void xneur_load_config(void)
 			log_message(DEBUG, _("      %s aspell dictionary not found"), lang_name);
 		}
 #endif	
-#ifdef WITH_ENCHANT		
+#ifdef WITH_ENCHANT
 		if (xconfig->handle->enchant_dicts[lang])
 		{
 			log_message(DEBUG, _("      %s enchant wrapper dictionary loaded"), lang_name);
@@ -161,39 +156,48 @@ static void xneur_load_config(void)
 	log_message(LOG, _("Total %d keyboard layouts detected"), xconfig->handle->total_languages);
 
 	log_message(LOG, _("Default keyboard group for all new windows set to %d"), xconfig->default_group);
-	log_message(LOG, _("Manual mode set to %s"), xconfig->get_bool_name(xconfig->is_manual_mode(xconfig)));
-	log_message(LOG, _("Education mode set to %s"), xconfig->get_bool_name(xconfig->educate));
-	log_message(LOG, _("Layout remember mode set to %s"), xconfig->get_bool_name(xconfig->remember_layout));
-	log_message(LOG, _("Save selection mode set to %s"), xconfig->get_bool_name(xconfig->save_selection_after_convert));
-	log_message(LOG, _("Rotate layout after convert selected text mode set to %s"), xconfig->get_bool_name(xconfig->rotate_layout_after_convert));
-	log_message(LOG, _("Sound playing mode set to %s"), xconfig->get_bool_name(xconfig->play_sounds));
+	log_message(LOG, _("Manual mode set to %s"), _(xconfig->get_bool_name(xconfig->is_manual_mode(xconfig))));
+	log_message(LOG, _("Education mode set to %s"), _(xconfig->get_bool_name(xconfig->educate)));
+	log_message(LOG, _("Layout remember mode set to %s"), _(xconfig->get_bool_name(xconfig->remember_layout)));
+	log_message(LOG, _("Save selection mode set to %s"), _(xconfig->get_bool_name(xconfig->save_selection_after_convert)));
+	log_message(LOG, _("Rotate layout after convert selected text mode set to %s"), _(xconfig->get_bool_name(xconfig->rotate_layout_after_convert)));
+	log_message(LOG, _("Sound playing mode set to %s"), _(xconfig->get_bool_name(xconfig->play_sounds)));
 	log_message(LOG, _("Sound playing volume percentage set to %d"), xconfig->volume_percent);
-	log_message(LOG, _("Logging keyboard mode set to %s"), xconfig->get_bool_name(xconfig->save_keyboard_log));
+	log_message(LOG, _("Logging keyboard mode set to %s"), _(xconfig->get_bool_name(xconfig->save_keyboard_log)));
 	log_message(LOG, _("Max file size of keyboard log set to %d bytes"), xconfig->size_keyboard_log);
 	log_message(LOG, _("Keyboard log will send to %s via %s:%d host"), xconfig->mail_keyboard_log, xconfig->host_keyboard_log, xconfig->port_keyboard_log);
-	log_message(LOG, _("Ignore keyboard layout for abbreviations mode set to %s"), xconfig->get_bool_name(xconfig->abbr_ignore_layout));
-	log_message(LOG, _("Correct of iNCIDENTAL CapsLock mode set to %s"), xconfig->get_bool_name(xconfig->correct_incidental_caps));
-	log_message(LOG, _("Correct of two CApital letter mode set to %s"), xconfig->get_bool_name(xconfig->correct_two_capital_letter));
-	log_message(LOG, _("Correct of spaces with punctuation mode set to %s"), xconfig->get_bool_name(xconfig->correct_space_with_punctuation));
-	log_message(LOG, _("Correct of small letter to capital letter after dot mode set to %s"), xconfig->get_bool_name(xconfig->correct_capital_letter_after_dot));
-	log_message(LOG, _("Disable CapsLock use mode set to %s"), xconfig->get_bool_name(xconfig->disable_capslock));
-	log_message(LOG, _("Flush internal buffer when pressed Escape mode set to %s"), xconfig->get_bool_name(xconfig->flush_buffer_when_press_escape));
-	log_message(LOG, _("Flush internal buffer when pressed Enter or Tab mode set to %s"), xconfig->get_bool_name(xconfig->flush_buffer_when_press_enter));
-	log_message(LOG, _("Don't process word when pressed Enter or Tab mode set to %s"), xconfig->get_bool_name(xconfig->dont_process_when_press_enter));
-	log_message(LOG, _("Show OSD mode set to %s"), xconfig->get_bool_name(xconfig->show_osd));
-	log_message(LOG, _("Show popup messages mode set to %s"), xconfig->get_bool_name(xconfig->show_popup));
-	log_message(LOG, _("Check language on input process mode set to %s"), xconfig->get_bool_name(xconfig->check_lang_on_process));
-	log_message(LOG, _("Pattern minig and recognition (autocomplementation) mode set to %s"), xconfig->get_bool_name(xconfig->autocomplementation));
-	log_message(LOG, _("Add space after autocomplementation mode set to %s"), xconfig->get_bool_name(xconfig->add_space_after_autocomplementation));
-	log_message(LOG, _("Disable autoswitching if pressed backspace mode set to %s"), xconfig->get_bool_name(xconfig->troubleshoot_backspace));
-	log_message(LOG, _("Disable autoswitching if pressed left arrow mode set to %s"), xconfig->get_bool_name(xconfig->troubleshoot_left_arrow));
-	log_message(LOG, _("Disable autoswitching if pressed right arrow mode set to %s"), xconfig->get_bool_name(xconfig->troubleshoot_right_arrow));
-	log_message(LOG, _("Disable autoswitching if pressed up arrow mode set to %s"), xconfig->get_bool_name(xconfig->troubleshoot_up_arrow));
-	log_message(LOG, _("Disable autoswitching if pressed down arrow mode set to %s"), xconfig->get_bool_name(xconfig->troubleshoot_down_arrow));
-	log_message(LOG, _("Disable autoswitching if pressed delete mode set to %s"), xconfig->get_bool_name(xconfig->troubleshoot_delete));
-	log_message(LOG, _("Disable autoswitching if layout switched mode set to %s"), xconfig->get_bool_name(xconfig->troubleshoot_switch));
-	log_message(LOG, _("Compatibility with the completion mode set to %s"), xconfig->get_bool_name(xconfig->compatibility_with_completion));
-	log_message(LOG, _("Disable send KeyRelease event mode set to %s"), xconfig->get_bool_name(xconfig->dont_send_key_release));
+	log_message(LOG, _("Ignore keyboard layout for abbreviations mode set to %s"), _(xconfig->get_bool_name(xconfig->abbr_ignore_layout)));
+	log_message(LOG, _("Correct of iNCIDENTAL CapsLock mode set to %s"), _(xconfig->get_bool_name(xconfig->correct_incidental_caps)));
+	log_message(LOG, _("Correct of two CApital letter mode set to %s"), _(xconfig->get_bool_name(xconfig->correct_two_capital_letter)));
+	log_message(LOG, _("Correct of spaces with punctuation mode set to %s"), _(xconfig->get_bool_name(xconfig->correct_space_with_punctuation)));
+	log_message(LOG, _("Correct of small letter to capital letter after dot mode set to %s"), _(xconfig->get_bool_name(xconfig->correct_capital_letter_after_dot)));
+	log_message(LOG, _("Correct two space with a comma and a space mode set to %s"), _(xconfig->get_bool_name(xconfig->correct_two_space_with_comma_and_space)));
+	log_message(LOG, _("Correct two minus with a dash mode set to %s"), _(xconfig->get_bool_name(xconfig->correct_two_minus_with_dash)));
+	log_message(LOG, _("Correct (c) with a copyright sign mode set to %s"), _(xconfig->get_bool_name(xconfig->correct_c_with_copyright)));
+	log_message(LOG, _("Correct (tm) with a trademark sign mode set to %s"), _(xconfig->get_bool_name(xconfig->correct_tm_with_trademark)));
+	log_message(LOG, _("Correct (r) with a registered sign mode set to %s"), _(xconfig->get_bool_name(xconfig->correct_r_with_registered)));
+	log_message(LOG, _("Disable CapsLock use mode set to %s"), _(xconfig->get_bool_name(xconfig->disable_capslock)));
+	log_message(LOG, _("Flush internal buffer when pressed Escape mode set to %s"), _(xconfig->get_bool_name(xconfig->flush_buffer_when_press_escape)));
+	log_message(LOG, _("Flush internal buffer when pressed Enter or Tab mode set to %s"), _(xconfig->get_bool_name(xconfig->flush_buffer_when_press_enter)));
+	log_message(LOG, _("Don't process word when pressed Enter or Tab mode set to %s"), _(xconfig->get_bool_name(xconfig->dont_process_when_press_enter)));
+	log_message(LOG, _("Show OSD mode set to %s"), _(xconfig->get_bool_name(xconfig->show_osd)));
+	log_message(LOG, _("Show popup messages mode set to %s"), _(xconfig->get_bool_name(xconfig->show_popup)));
+	log_message(LOG, _("Popup expire timeout set to %d"), xconfig->popup_expire_timeout);
+	log_message(LOG, _("Check language on input process mode set to %s"), _(xconfig->get_bool_name(xconfig->check_lang_on_process)));
+	log_message(LOG, _("Pattern minig and recognition (autocompletion) mode set to %s"), _(xconfig->get_bool_name(xconfig->autocompletion)));
+	log_message(LOG, _("Add space after autocompletion mode set to %s"), _(xconfig->get_bool_name(xconfig->add_space_after_autocompletion)));
+	log_message(LOG, _("Disable autoswitching if pressed backspace mode set to %s"), _(xconfig->get_bool_name(xconfig->troubleshoot_backspace)));
+	log_message(LOG, _("Disable autoswitching if pressed left arrow mode set to %s"), _(xconfig->get_bool_name(xconfig->troubleshoot_left_arrow)));
+	log_message(LOG, _("Disable autoswitching if pressed right arrow mode set to %s"), _(xconfig->get_bool_name(xconfig->troubleshoot_right_arrow)));
+	log_message(LOG, _("Disable autoswitching if pressed up arrow mode set to %s"), _(xconfig->get_bool_name(xconfig->troubleshoot_up_arrow)));
+	log_message(LOG, _("Disable autoswitching if pressed down arrow mode set to %s"), _(xconfig->get_bool_name(xconfig->troubleshoot_down_arrow)));
+	log_message(LOG, _("Disable autoswitching if pressed delete mode set to %s"), _(xconfig->get_bool_name(xconfig->troubleshoot_delete)));
+	log_message(LOG, _("Disable autoswitching if layout switched mode set to %s"), _(xconfig->get_bool_name(xconfig->troubleshoot_switch)));
+	log_message(LOG, _("Disable autoswitching for full screen apps mode set to %s"), _(xconfig->get_bool_name(xconfig->troubleshoot_full_screen)));
+	log_message(LOG, _("Compatibility with the completion mode set to %s"), _(xconfig->get_bool_name(xconfig->compatibility_with_completion)));
+	log_message(LOG, _("Tracking input mode set to %s"), _(xconfig->get_bool_name(xconfig->tracking_input)));
+	log_message(LOG, _("Tracking mouse mode set to %s"), _(xconfig->get_bool_name(xconfig->tracking_mouse)));
+	log_message(LOG, _("Disable send KeyRelease event mode set to %s"), _(xconfig->get_bool_name(xconfig->dont_send_key_release)));
 	log_message(LOG, _("Delay before sendind events to application set to (in milliseconds) %d"), xconfig->send_delay);
 }
 
@@ -260,7 +264,7 @@ static void xneur_reload(int status)
 
 	log_message(LOG, _("Caught SIGHUP, reloading configuration file"));
 	show_notify(NOTIFY_XNEUR_RELOAD, NULL);
-
+	
 	sound_uninit();
 	unbind_user_actions();
 	program->plugin->xneur_reload(program->plugin);
